package vmsSdk

import (
	"bytes"
	"encoding/json"
	"fmt"
	"github.com/baidubce/bce-sdk-go/auth"
	bcehttp "github.com/baidubce/bce-sdk-go/http"
	"io/ioutil"
	"net/http"
)

type LaunchCallClient struct {
	TemplateId    string
	MessageVarStr string
	CalledNum     string
	Type          int
	UserDefineTag string
	PlayTimes     int
	VoiceVolume   int
	Ak            string
	Sk            string
}

func (vmsClient *LaunchCallClient) LaunchCallToVms() (error, int, string, string) {
	client := &http.Client{}
	url := "https://vms.baidubce.com/v1/call"
	bodyStr, err := json.Marshal(getLaunchCallReqBody(vmsClient))
	if err != nil {
		fmt.Println("Marshal reqBody to json string fail, error is: %s", err.Error())
		return err, -1, "", ""
	}
	body := bytes.NewReader([]byte(bodyStr))
	req, err := http.NewRequest("POST", url, body)
	if err != nil {
		fmt.Println("New request fail, err is: %s", err.Error())
		return err, -1, "", ""
	} else {
		req.Header.Add("host", "vms.baidubce.com")
		req.Header.Set("Content-type", "application/json; charset=utf-8")
		option := &auth.SignOptions{
			HeadersToSign: auth.DEFAULT_HEADERS_TO_SIGN,
			ExpireSeconds: auth.DEFAULT_EXPIRE_SECONDS,
		}
		req.Header.Add(bcehttp.AUTHORIZATION, getAuthorizationString(getCredentials(vmsClient), option))
		resp, err := client.Do(req)
		if err != nil {
			fmt.Println("send http req to bce vms fail, error is: %s", err.Error())
			return err, -1, "", ""
		} else {
			responseBody, err := ioutil.ReadAll(resp.Body)
			if err != nil {
				fmt.Println("read resp fail, err is: %s", err.Error())
				return err, -1, "", ""
			} else {
				return nil, resp.StatusCode, resp.Status, string(responseBody)
			}
		}
	}
}

type CallByBceVmsReqBody struct {
	TemplateId    string `json:"voiceMessageTemplateId,omitempty"`
	MessageVarStr string `json:"voiceMessageVar,omitempty"`
	CalledNum     string `json:"calledNumber,omitempty"`
	Type          int    `json:"type,omitempty"`
	UserDefineTag string `json:"userDefinedTag,omitempty"`
	PlayTimes     int    `json:"playTimes,omitempty"`
	VoiceVolume   int    `json:"voiceVolume,omitemtpy"`
}

func getCredentials(client *LaunchCallClient) *auth.BceCredentials {
	credentials := &auth.BceCredentials{
		AccessKeyId:     client.Ak,
		SecretAccessKey: client.Sk,
	}
	return credentials
}
func getLaunchCallReqBody(client *LaunchCallClient) *CallByBceVmsReqBody {
	reqBody := &CallByBceVmsReqBody{}
	reqBody.TemplateId = client.TemplateId
	reqBody.MessageVarStr = client.MessageVarStr
	reqBody.CalledNum = client.CalledNum
	reqBody.Type = 0
	reqBody.UserDefineTag = client.UserDefineTag
	if client.PlayTimes == 0 {
		reqBody.PlayTimes = 1
	} else {
		reqBody.PlayTimes = client.PlayTimes
	}
	if client.VoiceVolume == 0 {
		reqBody.VoiceVolume = 20
	} else {
		reqBody.VoiceVolume = client.VoiceVolume
	}
	return reqBody
}
func getAuthorizationString(credentials *auth.BceCredentials, option *auth.SignOptions) string {
	request := &bcehttp.Request{}
	request.SetHost("vms.baidubce.com")

	request.SetUri("/v1/call")
	request.SetMethod("POST")
	request.SetHeader("host", "vms.baidubce.com")
	request.SetHeader("Content-type", "application/json; charset=utf-8")
	bce := &auth.BceV1Signer{}
	bce.Sign(request, credentials, option)
	return request.Header(bcehttp.AUTHORIZATION)
}
